<?php
/**
 * Copyright since 2022 PayGreen
 * NOTICE OF LICENSE
 * This source file is subject to the MIT License X11
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/mit-license.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@paygreen.fr so we can send you a copy immediately.
 * @author    PayGreen <contact@paygreen.fr>
 * @copyright 2014 - 2022 Watt Is It
 * @license   https://opensource.org/licenses/mit-license.php MIT License X11
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once __DIR__ . '/../../classes/PaygreenPaymentBuyerModel.php';
require_once __DIR__ . '/../../src/PaygreenPaymentPaymentOrderCreationException.php';
require_once __DIR__ . '/../../src/PaygreenPaymentUtils.php';
require_once __DIR__ . '/../../src/PaygreenPaymentOrderHelper.php';

use Paygreen\Sdk\Payment\V3\Enum\DomainEnum as SdkDomainEnum;
use Paygreen\Sdk\Payment\V3\Enum\StatusEnum as SdkStatusEnum;
use Paygreen\Sdk\Payment\V3\Model\Address as SdkAddressModel;
use Paygreen\Sdk\Payment\V3\Model\Buyer as SdkBuyerModel;
use Paygreen\Sdk\Payment\V3\Model\PaymentOrder as SdkPaymentOrderModel;

class PaygreenPaymentCreatePaymentOrderModuleFrontController extends ModuleFrontController
{
    /**
     * @return false
     * @throws PrestaShopException
     */
    public function postProcess()
    {
        try {
            $cart = new Cart((int) Context::getContext()->cart->id);

            /** @phpstan-ignore-next-line */
            $customer = new Customer($cart->id_customer);

            if (!Validate::isLoadedObject($cart) || !Validate::isLoadedObject($customer)) {
                return $this->displayError($this->module->l('Invalid cart data'));
            }

            if (!$cart->id_address_delivery || !$cart->id_address_invoice) {
                return $this->displayError($this->module->l('Missing shipping data'));
            }

            $returnUrl = $this->context->link->getModuleLink(
                $this->module->name,
                'return',
                [
                    'cart_id' => $cart->id,
                    'secure_key' => $customer->secure_key,
                ]
            );

            $cancelUrl = $this->context->link->getPageLink('order');

            $localShippingAddress = new Address((int) $cart->id_address_delivery);
            $localBillingAddress = new Address((int) $cart->id_address_invoice);

            if (!Validate::isLoadedObject($localShippingAddress) || !Validate::isLoadedObject($localBillingAddress)) {
                return $this->displayError($this->module->l('Invalid shipping data'));
            }

            // @phpstan-ignore-next-line
            $paygreenBuyer = new PaygreenPaymentBuyerModel();
            $paygreenBuyer = $paygreenBuyer->getByCustomerId($customer->id); /** @phpstan-ignore-line */
            $buyer = new SdkBuyerModel();
            $buyer->setReference('cus-' . $customer->id);
            $buyer->setEmail($customer->email);
            $buyer->setFirstname($customer->firstname);
            $buyer->setLastname($customer->lastname);

            $shippingAddress = new SdkAddressModel();
            $shippingAddress->setStreetLineOne($localShippingAddress->address1);
            $shippingAddress->setStreetLineTwo($localShippingAddress->address2);
            $shippingAddress->setCity($localShippingAddress->city);
            $shippingAddress->setCountryCode(strtoupper($this->getCountryCode($localShippingAddress->id_country)));
            $shippingAddress->setPostalCode($localShippingAddress->postcode);

            $billingAddress = new SdkAddressModel();
            $billingAddress->setStreetLineOne($localBillingAddress->address1);
            $billingAddress->setStreetLineTwo($localBillingAddress->address2);
            $billingAddress->setCity($localBillingAddress->city);
            $billingAddress->setCountryCode(strtoupper($this->getCountryCode($localBillingAddress->id_country)));
            $billingAddress->setPostalCode($localBillingAddress->postcode);

            $buyer->setBillingAddress($billingAddress);

            if (!empty($paygreenBuyer->getBuyerId()) && preg_match('/^buy_/', $paygreenBuyer->getBuyerId())) {
                // @phpstan-ignore-next-line
                $response = PaygreenPaymentUtils::getPaygreenClient()->getBuyer($paygreenBuyer->getBuyerId());

                if (200 === $response->getStatusCode()) {
                    $buyer->setId($paygreenBuyer->getBuyerId());

                    $customerUpdatedAt = (new DateTime($customer->date_upd))->getTimestamp();

                    if ($paygreenBuyer->getUpdatedAt() < $customerUpdatedAt) {
                        // @phpstan-ignore-next-line
                        $response = PaygreenPaymentUtils::getPaygreenClient()->updateBuyer($buyer);

                        if (200 === $response->getStatusCode()) {
                            $paygreenBuyer->setUpdatedAt((new DateTime())->getTimestamp());
                            $paygreenBuyer->save();
                        } else {
                            // @phpstan-ignore-next-line
                            PaygreenPaymentUtils::addLog('PaygreenPaymentCreatePaymentOrderModuleFrontController::postProcess - Failed to update buyer information. Response: ' . print_r($response->getBody()->getContents(), true), 3);
                        }
                    }
                }
            }

            $paymentOrderReference = 'ord-' . (new DateTime())->getTimestamp() . '-' . $cart->id . '-1';

            $shopGroupId = PaygreenPayment::getShopGroupIdContext(); /** @phpstan-ignore-line */
            $shopId = PaygreenPayment::getShopIdContext(); // @phpstan-ignore-line

            $paymentOrder = new SdkPaymentOrderModel();
            $paymentOrder->setAutoCapture(true);
            $paymentOrder->setMetadata([
                'cart_id' => $cart->id,
                'secure_key' => $customer->secure_key,
                'shop_id' => $shopId,
            ]);
            $paymentOrder->setEligibleAmounts([
                SdkDomainEnum::FOOD => $this->getEligibleAmount($cart, 'FOOD'),
                SdkDomainEnum::TRAVEL => $this->getEligibleAmount($cart, 'TRAVEL'),
            ]);
            $paymentOrder->setShippingAddress($shippingAddress);
            $paymentOrder->setReference($paymentOrderReference);
            // @phpstan-ignore-next-line
            $paymentOrder->setAmount((int) Tools::ps_round(PaygreenPaymentOrderHelper::getOrderAmount($cart) * 100));
            $paymentOrder->setCurrency('eur');
            $paymentOrder->setBuyer($buyer);
            $paymentOrder->setReturnUrl($returnUrl);
            $paymentOrder->setCancelUrl($cancelUrl);

            if (null !== Configuration::get('PAYGREEN_PAYMENT_SUB_SHOP_ID', null, $shopGroupId, $shopId)) {
                $paymentOrder->setShopId(Configuration::get('PAYGREEN_PAYMENT_SUB_SHOP_ID', null, $shopGroupId, $shopId));
            }

            $environment = Configuration::get('PAYGREEN_PAYMENT_ENVIRONMENT', null, $shopGroupId, $shopId);

            switch ($environment) {
                case 'RECETTE':
                case 'SANDBOX':
                    $result = false;

                    do {
                        // @phpstan-ignore-next-line
                        $response = PaygreenPaymentUtils::getPaygreenClient()->createPaymentOrder($paymentOrder);
                        $responseData = json_decode($response->getBody()->getContents());

                        if (200 === $response->getStatusCode()) {
                            // @phpstan-ignore-next-line
                            PaygreenPaymentUtils::addLog('PaygreenPaymentCreatePaymentOrderModuleFrontController::postProcess - ' . $responseData->data->id . ' - Payment order successfully created');
                            $this->createPaygreenBuyer($responseData->data->buyer->id, $customer->id);
                            $result = true;
                            Tools::redirect($responseData->data->hosted_payment_url);
                        } elseif (422 === $response->getStatusCode()) {
                            $paymentOrders = $this->getPaymentOrders($paymentOrderReference);

                            foreach ($paymentOrders as $item) {
                                switch ($item->status) {
                                    case 'payment_order.' . SdkStatusEnum::PENDING:
                                        if (!$this->cancelPaymentOrder($item->id)) {
                                            $paymentOrder->setReference($this->regeneratePOReference($paymentOrder->getReference()));
                                        }

                                        break;
                                    default:
                                        $paymentOrder->setReference($this->regeneratePOReference($paymentOrder->getReference()));

                                        break;
                                }
                            }
                        } else {
                            // @phpstan-ignore-next-line
                            PaygreenPaymentUtils::addLog('PaygreenPaymentCreatePaymentOrderModuleFrontController::postProcess - Failed to create payment order. Response: ' . print_r($responseData, true), 3);

                            // @phpstan-ignore-next-line
                            throw new PaygreenPaymentPaymentOrderCreationException();
                        }
                    } while (!$result);

                    break;
                case 'PRODUCTION':
                    // @phpstan-ignore-next-line
                    $response = PaygreenPaymentUtils::getPaygreenClient()->createPaymentOrder($paymentOrder);
                    $responseData = json_decode($response->getBody()->getContents());

                    if (200 === $response->getStatusCode()) {
                        // @phpstan-ignore-next-line
                        PaygreenPaymentUtils::addLog('PaygreenPaymentCreatePaymentOrderModuleFrontController::postProcess - ' . $responseData->data->id . ' - Payment order successfully created');
                        $this->createPaygreenBuyer($responseData->data->buyer->id, $customer->id);
                        Tools::redirect($responseData->data->hosted_payment_url);
                    } elseif (422 === $response->getStatusCode()) {
                        $paymentOrders = $this->getPaymentOrders($paymentOrderReference);

                        foreach ($paymentOrders as $item) {
                            switch ($item->status) {
                                case 'payment_order.' . SdkStatusEnum::PENDING:
                                    if ($this->cancelPaymentOrder($item->id)) {
                                        // @phpstan-ignore-next-line
                                        $response = PaygreenPaymentUtils::getPaygreenClient()->createPaymentOrder($paymentOrder);
                                        $responseData = json_decode($response->getBody()->getContents());

                                        if (200 !== $response->getStatusCode()) {
                                            // @phpstan-ignore-next-line
                                            PaygreenPaymentUtils::addLog('PaygreenPaymentCreatePaymentOrderModuleFrontController::postProcess - Failed to create payment order. Response: ' . print_r($responseData, true), 3);

                                            // @phpstan-ignore-next-line
                                            throw new PaygreenPaymentPaymentOrderCreationException();
                                        }

                                        // @phpstan-ignore-next-line
                                        PaygreenPaymentUtils::addLog('PaygreenPaymentCreatePaymentOrderModuleFrontController::postProcess - ' . $responseData->data->id . ' - Payment order successfully created');
                                        $this->createPaygreenBuyer($responseData->data->buyer->id, $customer->id);
                                        Tools::redirect($responseData->data->hosted_payment_url);
                                    }

                                    break;
                                case 'payment_order.' . SdkStatusEnum::REFUSED:
                                    $result = false;

                                    do {
                                        $paymentOrder->setReference($this->regeneratePOReference($paymentOrder->getReference()));
                                        // @phpstan-ignore-next-line
                                        $response = PaygreenPaymentUtils::getPaygreenClient()->createPaymentOrder($paymentOrder);
                                        $responseData = json_decode($response->getBody()->getContents());

                                        if (200 === $response->getStatusCode()) {
                                            // @phpstan-ignore-next-line
                                            PaygreenPaymentUtils::addLog('PaygreenPaymentCreatePaymentOrderModuleFrontController::postProcess - ' . $responseData->data->id . ' - Payment order successfully created');
                                            $this->createPaygreenBuyer($responseData->data->buyer->id, $customer->id);
                                            $result = true;
                                            Tools::redirect($responseData->data->hosted_payment_url);
                                        }
                                    } while (!$result);
                                    break;
                            }
                        }
                    } else {
                        // @phpstan-ignore-next-line
                        PaygreenPaymentUtils::addLog('PaygreenPaymentCreatePaymentOrderModuleFrontController::postProcess - Failed to create payment order. Response: ' . print_r($responseData, true), 3);

                        // @phpstan-ignore-next-line
                        throw new PaygreenPaymentPaymentOrderCreationException();
                    }

                    break;
                default:
                    throw new Exception('Invalid environment.');
            }
            // @phpstan-ignore-next-line
        } catch (PaygreenPaymentPaymentOrderCreationException $exception) {
            return $this->displayError($this->module->l('An error occurred during payment creation. Please check the information you have provided. Check that your information does not contain any special characters and is at least two characters long.'));
        } catch (Exception $exception) {
            return $this->displayError($this->module->l('An error occurred. Please contact the merchant to have more information.'));
        }

        return $this->displayError($this->module->l('An error occurred. Please contact the merchant to have more information.'));
    }

    /**
     * @return bool
     */
    public function setMedia()
    {
        parent::setMedia();

        // @phpstan-ignore-next-line
        $this->context->controller->registerStylesheet(
            'error',
            'modules/' . $this->module->name . '/views/css/error.css',
            ['priority' => 150]
        );

        return true;
    }

    /**
     * @param string $message
     * @return false
     * @throws PrestaShopException
     */
    private function displayError($message)
    {
        $this->context->smarty->assign([
            'message' => $message,
            'checkout_url' => $this->context->link->getPageLink('order', true, null, [
                'step' => 3,
            ]),
        ]);

        $this->setTemplate('module:' . $this->module->name . '/views/templates/front/error_17.tpl');

        return false;
    }

    /**
     * @param Cart $cart
     * @param string $type
     * @return int
     * @throws Exception
     */
    private function getEligibleAmount(Cart $cart, $type)
    {
        // @phpstan-ignore-next-line
        $totalAmount = (int) Tools::ps_round(PaygreenPaymentOrderHelper::getOrderAmount($cart) * 100);
        $eligibleAmount = 0;

        $shopGroupId = PaygreenPayment::getShopGroupIdContext(); /** @phpstan-ignore-line */
        $shopId = PaygreenPayment::getShopIdContext(); /** @phpstan-ignore-line */
        $eligibleCategories = json_decode(Configuration::get('PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type, null, $shopGroupId, $shopId), true);
        $eligibleCategoriesAllowAll = (bool) Configuration::get('PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type . '_ALLOW_ALL', null, $shopGroupId, $shopId);
        $shippingCostExclusion = (bool) Configuration::get('PAYGREEN_PAYMENT_SHIPPING_COST_EXCLUSION', null, $shopGroupId, $shopId);

        if (!$eligibleCategoriesAllowAll && (null === $eligibleCategories || !$eligibleCategories['PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type])) {
            return 0;
        }

        $products = $cart->getProducts();

        foreach ($products as $product) {
            if ($eligibleCategoriesAllowAll || $this->isProductEligible($product, $eligibleCategories['PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type])) {
                $eligibleAmount += ($product['price_wt'] * $product['quantity']);
            }
        }

        $eligibleAmount = (int) Tools::ps_round(Tools::ps_round($eligibleAmount, 2) * 100);

        if (!$shippingCostExclusion) {
            $eligibleAmount += (int) (Tools::ps_round($cart->getTotalShippingCost(), 2) * 100);
        }

        if ($eligibleAmount > $totalAmount) {
            $eligibleAmount = $totalAmount;
        }

        return $eligibleAmount;
    }

    /**
     * Check if a product is eligible for a meal or vacation voucher.
     * @return bool
     */
    private function isProductEligible(array $product, array $eligibleCategories)
    {
        $category = new Category($product['id_category_default']);

        $isEligibleCategory = $this->isEligibleCategory($category, $eligibleCategories);

        if ($isEligibleCategory) {
            return true;
        }

        if ($category->level_depth > 0) {
            $parentCategories = $category->getAllParents()->getResults();

            foreach ($parentCategories as $category) {
                $isEligibleCategory = $this->isEligibleCategory($category, $eligibleCategories);

                if ($isEligibleCategory) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * @param Category $category
     * @param array $eligibleCategories
     * @return bool
     */
    private function isEligibleCategory($category, array $eligibleCategories)
    {
        return in_array($category->id, $eligibleCategories);
    }

    /**
     * @param int $countryId
     * @return string
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    private function getCountryCode($countryId)
    {
        $country = new Country($countryId);

        return $country->iso_code;
    }

    /**
     * @param string $buyerId
     * @param int $customerId
     * @return void
     * @throws PrestaShopException
     */
    private function createPaygreenBuyer($buyerId, $customerId)
    {
        // @phpstan-ignore-next-line
        $paygreenBuyer = new PaygreenPaymentBuyerModel();
        $paygreenBuyer->getByBuyerId($buyerId);

        if (empty($paygreenBuyer->getCustomerId())) {
            $paygreenBuyer->setBuyerId($buyerId);
            $paygreenBuyer->setCustomerId($customerId); /** @phpstan-ignore-line */
            $timestamp = (new DateTime())->getTimestamp();
            $paygreenBuyer->setCreatedAt($timestamp);
            $paygreenBuyer->setUpdatedAt($timestamp);
            $paygreenBuyer->save();
        }
    }

    /**
     * @param string $paymentOrderId
     * @return bool
     * @throws Exception
     */
    private function cancelPaymentOrder($paymentOrderId)
    {
        // @phpstan-ignore-next-line
        $response = PaygreenPaymentUtils::getPaygreenClient()->cancelPaymentOrder($paymentOrderId);
        $responseData = json_decode($response->getBody()->getContents());

        if (200 !== $response->getStatusCode()) {
            // @phpstan-ignore-next-line
            PaygreenPaymentUtils::addLog('PaygreenPaymentCreatePaymentOrderModuleFrontController::cancelPaymentOrder - ' . $paymentOrderId . ' - Failed to cancel payment order. Response: ' . print_r($responseData, true), 3);

            return false;
        } else {
            // @phpstan-ignore-next-line
            PaygreenPaymentUtils::addLog('PaygreenPaymentCreatePaymentOrderModuleFrontController::postProcess - ' . $paymentOrderId . ' - Payment order successfully canceled');

            return true;
        }
    }

    /**
     * @param string $paymentOrderReference
     * @return mixed
     * @throws Exception
     */
    private function getPaymentOrders($paymentOrderReference)
    {
        // @phpstan-ignore-next-line
        $response = PaygreenPaymentUtils::getPaygreenClient()->listPaymentOrder($paymentOrderReference);
        $responseData = json_decode($response->getBody()->getContents());

        if (200 !== $response->getStatusCode()) {
            // @phpstan-ignore-next-line
            PaygreenPaymentUtils::addLog('PaygreenPaymentCreatePaymentOrderModuleFrontController::getPaymentOrders - Failed to fetch payment order "' . $paymentOrderReference . '". Response: ' . print_r($responseData, true), 3);

            throw new Exception('Failed to fetch payment order.');
        }

        return $responseData->data;
    }

    /**
     * @param string $reference
     * @return string
     */
    private function regeneratePOReference($reference)
    {
        $referenceParts = explode('-', $reference);
        $attempt = (int) end($referenceParts);
        unset($referenceParts[count($referenceParts) - 1]);

        return implode('-', $referenceParts) . '-' . ($attempt + 1);
    }
}
