<?php
/**
 * Copyright since 2022 PayGreen
 * NOTICE OF LICENSE
 * This source file is subject to the MIT License X11
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/mit-license.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@paygreen.fr so we can send you a copy immediately.
 * @author    PayGreen <contact@paygreen.fr>
 * @copyright 2014 - 2022 Watt Is It
 * @license   https://opensource.org/licenses/mit-license.php MIT License X11
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once __DIR__ . '/../../src/PaygreenPaymentUtils.php';
require_once __DIR__ . '/../../src/PaygreenPaymentLockHelper.php';
require_once __DIR__ . '/../../src/PaygreenPaymentOrderHelper.php';

class PaygreenPaymentReturnModuleFrontController extends ModuleFrontController
{
    /** @var string[] */
    private static $RETURN_PROCESS_MANDATORY_DATA = ['cart_id', 'secure_key', 'po_id'];

    /**
     * @return false|void
     * @throws PrestaShopException
     */
    public function postProcess()
    {
        // @phpstan-ignore-next-line
        PaygreenPaymentUtils::addLog('PaygreenPaymentReturnModuleFrontController::postProcess - start');

        foreach (self::$RETURN_PROCESS_MANDATORY_DATA as $name) {
            if (!Tools::isSubmit($name)) {
                // @phpstan-ignore-next-line
                PaygreenPaymentUtils::addLog('PaygreenPaymentReturnModuleFrontController::postProcess - Missing mandatory data : ' . $name, 3);
                Tools::redirect($this->context->link->getModuleLink($this->module->name, 'failure'));
            }
        }

        $poId = Tools::getValue('po_id');
        $cartId = Tools::getValue('cart_id');
        $secureKey = Tools::getValue('secure_key');

        // @phpstan-ignore-next-line
        PaygreenPaymentUtils::addLog('PaygreenPaymentReturnModuleFrontController::postProcess - ' . $poId . ' - processing');

        // @phpstan-ignore-next-line
        while (PaygreenPaymentLockHelper::isLocked($poId, 'return')) {
            // @phpstan-ignore-next-line
            sleep(PaygreenPaymentLockHelper::LOCK_WAITING);
        }

        // @phpstan-ignore-next-line
        PaygreenPaymentLockHelper::lock($poId, 'return');

        $cart = new Cart((int) $cartId);
        /** @phpstan-ignore-next-line */
        $customer = new Customer($cart->id_customer);

        if (!Validate::isLoadedObject($cart)) {
            // @phpstan-ignore-next-line
            PaygreenPaymentUtils::addLog('PaygreenPaymentReturnModuleFrontController::postProcess - Invalid cart', 3);
            // @phpstan-ignore-next-line
            PaygreenPaymentLockHelper::unlock($poId, 'return');
            Tools::redirect($this->context->link->getModuleLink($this->module->name, 'failure'));
        }

        if (!Validate::isLoadedObject($customer)) {
            // @phpstan-ignore-next-line
            PaygreenPaymentUtils::addLog('PaygreenPaymentReturnModuleFrontController::postProcess - Invalid customer', 3);
            // @phpstan-ignore-next-line
            PaygreenPaymentLockHelper::unlock($poId, 'return');
            Tools::redirect($this->context->link->getModuleLink($this->module->name, 'failure'));
        }

        if (0 === $cart->id_customer || 0 === $cart->id_address_delivery || 0 === $cart->id_address_invoice || !$this->module->active) {
            // @phpstan-ignore-next-line
            PaygreenPaymentLockHelper::unlock($poId, 'return');
            Tools::redirect($this->context->link->getModuleLink($this->module->name, 'failure'));
        }

        try {
            // @phpstan-ignore-next-line
            $poStatus = PaygreenPaymentUtils::getPOStatus($poId);

            if (!$poStatus) {
                // @phpstan-ignore-next-line
                PaygreenPaymentLockHelper::unlock($poId, 'return');
                Tools::redirect($this->context->link->getModuleLink($this->module->name, 'failure'));
            }

            if (!in_array($poStatus, ['authorized', 'successed', 'error'])) {
                // @phpstan-ignore-next-line
                PaygreenPaymentUtils::addLog('PaygreenPaymentReturnModuleFrontController::postProcess - ' . $poId . ' - Invalid payment order status : ' . $poStatus, 3);

                // @phpstan-ignore-next-line
                PaygreenPaymentLockHelper::unlock($poId, 'return');
                Tools::redirect($this->context->link->getModuleLink($this->module->name, 'failure'));
            }

            // @phpstan-ignore-next-line
            if (!PaygreenPaymentOrderHelper::isOrderExist($cartId)) {
                // @phpstan-ignore-next-line
                if (!PaygreenPaymentOrderHelper::createOrder($this->module, $cart, $secureKey, $poStatus, $poId, 'return')) {
                    // @phpstan-ignore-next-line
                    PaygreenPaymentUtils::addLog('PaygreenPaymentReturnModuleFrontController::postProcess - ' . $poId . ' - An error occurred while creating order for cart_id ' . $cartId, 4);
                    // @phpstan-ignore-next-line
                    PaygreenPaymentLockHelper::unlock($poId, 'return');
                    Tools::redirect($this->context->link->getModuleLink($this->module->name, 'failure'));
                } else {
                    // @phpstan-ignore-next-line
                    PaygreenPaymentUtils::addLog('PaygreenPaymentReturnModuleFrontController::postProcess - ' . $poId . ' - Order successfully created for cart_id ' . $cartId);
                }
            } else {
                // @phpstan-ignore-next-line
                PaygreenPaymentUtils::addLog('PaygreenPaymentReturnModuleFrontController::postProcess - ' . $poId . ' - Order already exist for cart ' . $cartId);
            }

            // @phpstan-ignore-next-line
            PaygreenPaymentLockHelper::unlock($poId, 'return');

            // @phpstan-ignore-next-line
            if (!$order = PaygreenPaymentOrderHelper::getOrder($cartId, $poId, 'return')) {
                Tools::redirect($this->context->link->getModuleLink($this->module->name, 'failure'));
            }

            // @phpstan-ignore-next-line
            PaygreenPaymentOrderHelper::createPaygreenOrder($order->id, $poId);

            if ($secureKey === $customer->secure_key) {
                switch ($poStatus) {
                    case 'authorized':
                    case 'successed':
                        // @phpstan-ignore-next-line
                        PaygreenPaymentUtils::addLog('PaygreenPaymentReturnModuleFrontController::postProcess - ' . $poId . ' - Handle payment order : ' . $poStatus);

                        // @phpstan-ignore-next-line
                        if (!PaygreenPaymentOrderHelper::isOrderPaid($order) && !PaygreenPaymentOrderHelper::isOrderCancel($order) && !PaygreenPaymentOrderHelper::isOrderError($order)) {
                            $order->setCurrentState((int) Configuration::get('PS_OS_PAYMENT'));
                            $order->save();
                        }

                        Tools::redirect('index.php?controller=order-confirmation&id_cart=' . $cartId . '&id_module=' . $this->module->id . '&id_order=' . $order->id . '&key=' . $secureKey);
                        break;
                    case 'error':
                        // @phpstan-ignore-next-line
                        PaygreenPaymentUtils::addLog('PaygreenPaymentReturnModuleFrontController::postProcess - ' . $poId . ' - Handle payment order : ' . $poStatus);

                        // @phpstan-ignore-next-line
                        if (!PaygreenPaymentOrderHelper::isOrderPaid($order) && !PaygreenPaymentOrderHelper::isOrderCancel($order)) {
                            $order->setCurrentState((int) Configuration::get('PS_OS_ERROR'));
                            $order->save();
                        }
                        break;
                }

                Tools::redirect($this->context->link->getModuleLink($this->module->name, 'failure'));
            } else {
                // @phpstan-ignore-next-line
                PaygreenPaymentUtils::addLog('PaygreenPaymentReturnModuleFrontController::postProcess - ' . $poId . ' - Provided secure key mismatch the customer secure key', 3);

                $this->displayError();

                return false;
            }
        } catch (Exception $e) {
            // @phpstan-ignore-next-line
            PaygreenPaymentUtils::addLog('PaygreenPaymentReturnModuleFrontController::postProcess - Exception - ' . preg_replace("/\n/", '<br>', (string) $e->getMessage() . '<br>' . $e->getTraceAsString()), 3);
            // @phpstan-ignore-next-line
            PaygreenPaymentLockHelper::unlock($poId, 'return');

            $this->displayError();

            return false;
        }
    }

    // @phpstan-ignore-next-line
    private function displayError()
    {
        $this->context->smarty->assign([
            'message' => $this->module->l('An error occurred. Please contact the merchant to have more information.'),
            'checkout_url' => $this->context->link->getPageLink('order', true, null, [
                'step' => 3,
            ]),
        ]);

        $this->setTemplate('module:' . $this->module->name . '/views/templates/front/error_17.tpl');
    }
}
