<?php
/**
 * Copyright since 2022 PayGreen
 * NOTICE OF LICENSE
 * This source file is subject to the MIT License X11
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/mit-license.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@paygreen.fr so we can send you a copy immediately.
 * @author    PayGreen <contact@paygreen.fr>
 * @copyright 2014 - 2022 Watt Is It
 * @license   https://opensource.org/licenses/mit-license.php MIT License X11
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once __DIR__ . '/../../src/PaygreenPaymentUtils.php';
require_once __DIR__ . '/../../src/PaygreenPaymentLockHelper.php';
require_once __DIR__ . '/../../src/PaygreenPaymentOrderHelper.php';

class PaygreenPaymentWebhookModuleFrontController extends ModuleFrontController
{
    /** @var string[] */
    private static $WEBHOOK_PROCESS_MANDATORY_METADATA = ['cart_id', 'shop_id', 'secure_key'];

    const VALIDATION_SUCCEEDED = 'validation_succeeded';
    const VALIDATION_FAILED = 'validation_failed';
    const VALIDATION_FAILED_EMPTY_HEADERS = 'empty_headers';
    const VALIDATION_FAILED_EMPTY_BODY = 'empty_body';
    const VALIDATION_FAILED_SIGNATURE_MISMATCH = 'signature_mismatch';
    const VALIDATION_FAILED_HMAC_KEY_NOT_FOUND = 'hmac_key_not_found';

    /**
     * @return void
     */
    public function postProcess()
    {
        try {
            // @phpstan-ignore-next-line
            PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::postProcess - Paygreen incoming webhook : ' . @Tools::file_get_contents('php://input'));
            $validationResult = $this->checkWebhookAuthenticity();

            if (self::VALIDATION_SUCCEEDED === $validationResult) {
                http_response_code($this->processWebhook());
                exit(0);
            }

            // @phpstan-ignore-next-line
            PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::postProcess - Invalid webhook detected : ' . $validationResult, 3);

            exit(0);
        } catch (Exception $exception) {
            // @phpstan-ignore-next-line
            PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::postProcess - An error occurred : ' . $exception->getMessage(), 3);
        }

        http_response_code(400);
        exit(1);
    }

    /**
     * Gets the incoming request headers. Some servers are not using
     * Apache and "getallheaders()" will not work so we may need to
     * build our own headers.
     *
     * @return array
     */
    public function get_request_headers()
    {
        if (!function_exists('getallheaders')) {
            $headers = [];

            foreach ($_SERVER as $name => $value) {
                if ('HTTP_' === substr($name, 0, 5)) {
                    $headers[str_replace(' ', '-', ucwords(strtolower(str_replace('_', ' ', substr($name, 5)))))] = $value;
                }
            }

            return $headers;
        }

        return getallheaders();
    }

    /**
     * @return int
     */
    protected function processWebhook()
    {
        $input = @Tools::file_get_contents('php://input');

        if (empty($input) || !is_string($input)) {
            // @phpstan-ignore-next-line
            PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - Missing data', 3);

            return 400;
        }

        $webhookData = json_decode($input, true);

        if (array_key_exists('id', $webhookData) && !empty($webhookData['id'])) {
            try {
                $objectType = explode('.', $webhookData['status'])[0];

                if ($objectType === 'payment_order') {
                    $poId = $webhookData['id'];
                    $metadata = $webhookData['metadata'];

                    foreach (self::$WEBHOOK_PROCESS_MANDATORY_METADATA as $name) {
                        if (!in_array($name, array_keys($metadata))) {
                            // @phpstan-ignore-next-line
                            PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $poId . ' - Missing mandatory metadata : ' . $name, 3);

                            return 400;
                        }
                    }

                    // @phpstan-ignore-next-line
                    $shopId = PaygreenPayment::getShopIdContext();

                    if ($metadata['shop_id'] !== $shopId) {
                        // @phpstan-ignore-next-line
                        PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $poId . ' - This notification does not concern the current shop, no further process required.');

                        return 200;
                    }

                    // @phpstan-ignore-next-line
                    if (!PaygreenPaymentLockHelper::isLocked($poId, 'webhook')) {
                        // @phpstan-ignore-next-line
                        PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $poId . ' - Notification process not locked, retry once to avoid duplicate orders');
                        sleep(1);
                    }

                    // @phpstan-ignore-next-line
                    if (PaygreenPaymentLockHelper::isLocked($poId, 'webhook')) {
                        // @phpstan-ignore-next-line
                        PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $poId . ' - Notification process locked, end of process');

                        return 422;
                    }

                    // @phpstan-ignore-next-line
                    PaygreenPaymentLockHelper::lock($poId, 'webhook');

                    // @phpstan-ignore-next-line
                    $poStatus = PaygreenPaymentUtils::getPOStatus($poId);

                    if (!$poStatus) {
                        // @phpstan-ignore-next-line
                        PaygreenPaymentLockHelper::unlock($poId, 'webhook');

                        return 400;
                    }

                    $cart = $this->getCart($metadata['cart_id']);

                    if ($cart !== null) {
                        $cartId = $cart->id;

                        switch ($poStatus) {
                            case 'authorized':
                            case 'successed':
                                // @phpstan-ignore-next-line
                                if (!PaygreenPaymentOrderHelper::isOrderExist($cartId)) {
                                    // @phpstan-ignore-next-line
                                    if (!PaygreenPaymentOrderHelper::createOrder($this->module, $cart, $metadata['secure_key'], $poStatus, $poId, 'webhook')) {
                                        // @phpstan-ignore-next-line
                                        PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $poId . ' - An error occurred while creating order for cart_id ' . $cartId, 4);
                                        // @phpstan-ignore-next-line
                                        PaygreenPaymentLockHelper::unlock($poId, 'webhook');

                                        return 400;
                                    } else {
                                        // @phpstan-ignore-next-line
                                        PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $poId . ' - Order successfully created for cart_id ' . $cartId);
                                    }
                                } else {
                                    // @phpstan-ignore-next-line
                                    PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $poId . ' - Order already exist for cart ' . $cartId);
                                }

                                // @phpstan-ignore-next-line
                                PaygreenPaymentLockHelper::unlock($poId, 'webhook');

                                // @phpstan-ignore-next-line
                                if (!$order = PaygreenPaymentOrderHelper::getOrder($cartId, $poId, 'webhook')) {
                                    return 400;
                                }

                                // @phpstan-ignore-next-line
                                PaygreenPaymentOrderHelper::createPaygreenOrder($order->id, $poId);

                                // @phpstan-ignore-next-line
                                if (!PaygreenPaymentOrderHelper::isOrderPaid($order) && !PaygreenPaymentOrderHelper::isOrderCancel($order) && !PaygreenPaymentOrderHelper::isOrderError($order)) {
                                    $order->setCurrentState((int) Configuration::get('PS_OS_PAYMENT'));
                                    $order->save();
                                }

                                break;

                            case 'canceled':
                            case 'expired':
                                // @phpstan-ignore-next-line
                                if (PaygreenPaymentOrderHelper::isOrderExist($cart->id)) {
                                    // @phpstan-ignore-next-line
                                    if (!$order = PaygreenPaymentOrderHelper::getOrder($cartId, $poId, 'webhook')) {
                                        // @phpstan-ignore-next-line
                                        PaygreenPaymentLockHelper::unlock($poId, 'webhook');

                                        return 400;
                                    }

                                    // @phpstan-ignore-next-line
                                    if (!PaygreenPaymentOrderHelper::isOrderPaid($order) && !PaygreenPaymentOrderHelper::isOrderCancel($order) && !PaygreenPaymentOrderHelper::isOrderError($order)) {
                                        $order->setCurrentState((int) Configuration::get('PS_OS_CANCELED'));
                                        $order->save();
                                    }
                                }

                                break;

                            case 'error':
                                // @phpstan-ignore-next-line
                                if (PaygreenPaymentOrderHelper::isOrderExist($cart->id)) {
                                    // @phpstan-ignore-next-line
                                    if (!$order = PaygreenPaymentOrderHelper::getOrder($cartId, $poId, 'webhook')) {
                                        // @phpstan-ignore-next-line
                                        PaygreenPaymentLockHelper::unlock($poId, 'webhook');

                                        return 400;
                                    }

                                    // @phpstan-ignore-next-line
                                    if (!PaygreenPaymentOrderHelper::isOrderPaid($order) && !PaygreenPaymentOrderHelper::isOrderCancel($order)) {
                                        $order->setCurrentState((int) Configuration::get('PS_OS_ERROR'));
                                        $order->save();
                                    }
                                }

                                break;

                            case 'refunded':
                                // @phpstan-ignore-next-line
                                if (PaygreenPaymentOrderHelper::isOrderExist($cart->id)) {
                                    // @phpstan-ignore-next-line
                                    if (!$order = PaygreenPaymentOrderHelper::getOrder($cartId, $poId, 'webhook')) {
                                        // @phpstan-ignore-next-line
                                        PaygreenPaymentLockHelper::unlock($poId, 'webhook');

                                        return 400;
                                    }

                                    $order->setCurrentState((int) Configuration::get('PS_OS_REFUND'));
                                    $order->save();
                                }

                                break;
                            default:
                                break;
                        }

                        // @phpstan-ignore-next-line
                        PaygreenPaymentLockHelper::unlock($poId, 'webhook');
                    } else {
                        // @phpstan-ignore-next-line
                        PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $poId . ' - Cart not found for cart id : ' . $metadata['cart_id'], 2);
                        // @phpstan-ignore-next-line
                        PaygreenPaymentLockHelper::unlock($poId, 'webhook');

                        return 400;
                    }
                } elseif ($objectType === 'operation') {
                    $operationId = $webhookData['id'];
                    $operationStatus = $webhookData['status'];

                    if ($operationStatus === 'operation.refund') {
                        // @phpstan-ignore-next-line
                        $client = PaygreenPaymentUtils::getPaygreenClient();
                        $response = $client->getOperation($operationId);

                        if ($response->getStatusCode() === 200) {
                            $operation = json_decode($response->getBody()->getContents(), true)['data'];

                            if ($operation['status'] === $operationStatus) {
                                $response = $client->getPaymentOrder($operation['payment_order_id']);

                                if ($response->getStatusCode() === 200) {
                                    $paymentOrder = json_decode($response->getBody()->getContents(), true)['data'];
                                    $metadata = $paymentOrder['metadata'];

                                    foreach (self::$WEBHOOK_PROCESS_MANDATORY_METADATA as $name) {
                                        if (!in_array($name, array_keys($metadata))) {
                                            // @phpstan-ignore-next-line
                                            PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $operationId . ' - Missing mandatory metadata : ' . $name, 3);

                                            return 400;
                                        }
                                    }

                                    // @phpstan-ignore-next-line
                                    $shopId = PaygreenPayment::getShopIdContext();

                                    if ($metadata['shop_id'] !== $shopId) {
                                        // @phpstan-ignore-next-line
                                        PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $operationId . ' - This notification does not concern the current shop, no further process required.');

                                        return 200;
                                    }

                                    if (!$cart = $this->getCart($metadata['cart_id'])) {
                                        // @phpstan-ignore-next-line
                                        PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $operationId . ' - Cart not found for cart id : ' . $metadata['cart_id'], 2);

                                        return 400;
                                    }

                                    // @phpstan-ignore-next-line
                                    if (!$order = PaygreenPaymentOrderHelper::getOrder($cart->id, $operation['payment_order_id'], 'webhook')) {
                                        return 400;
                                    }

                                    $this->createPaygreenOrderMessage(
                                        $order->id,
                                        $this->formatAmount($webhookData['amount']) . '€',
                                        $operationStatus
                                    );
                                } else {
                                    // @phpstan-ignore-next-line
                                    PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $operationId . ' - Payment order not found for payment_order_id : ' . $operation['payment_order_id'], 3);

                                    return 400;
                                }
                            } else {
                                // @phpstan-ignore-next-line
                                PaygreenPaymentUtils::addLog(
                                    'PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $operationId . ' - Status mismatch : (webhook) ' . $operationStatus . ' - (api) ' . $operation['status'],
                                    3
                                );

                                return 400;
                            }
                        } else {
                            // @phpstan-ignore-next-line
                            PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - ' . $operationId . ' - Operation not found', 3);

                            return 400;
                        }
                    } else {
                        return 400;
                    }
                }
            } catch (Exception $exception) {
                // @phpstan-ignore-next-line
                PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController - Exception - ' . preg_replace("/\n/", '<br>', (string) $exception->getMessage() . '<br>' . $exception->getTraceAsString()), 3);
                // @phpstan-ignore-next-line
                PaygreenPaymentLockHelper::unlock($poId, 'webhook');

                return 400;
            }
        } else {
            // @phpstan-ignore-next-line
            PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - Missing webhook id', 3);

            return 400;
        }

        // @phpstan-ignore-next-line
        PaygreenPaymentUtils::addLog('PaygreenPaymentWebhookModuleFrontController::processWebhook - end');

        return 200;
    }

    /**
     * @return string
     */
    private function checkWebhookAuthenticity()
    {
        if (!isset($_SERVER['REQUEST_METHOD'])
            || ('POST' !== $_SERVER['REQUEST_METHOD'])
        ) {
            return self::VALIDATION_FAILED;
        }

        $requestBody = @Tools::file_get_contents('php://input');
        $requestHeaders = array_change_key_case($this->get_request_headers(), CASE_UPPER);

        // Validate it to make sure it is legit.
        return $this->checkRequestAuthenticity($requestHeaders, $requestBody);
    }

    /**
     * @param array $headers
     * @param bool|string $body
     *
     * @return string
     */
    private function checkRequestAuthenticity($headers, $body)
    {
        if (empty($headers)) {
            return self::VALIDATION_FAILED_EMPTY_HEADERS;
        }

        if (empty($body)) {
            return self::VALIDATION_FAILED_EMPTY_BODY;
        }

        $shopGroupId = PaygreenPayment::getShopGroupIdContext(); /** @phpstan-ignore-line */
        $shopId = PaygreenPayment::getShopIdContext(); /** @phpstan-ignore-line */
        $hmacKey = Configuration::get('PAYGREEN_LISTENER_HMAC_KEY', null, $shopGroupId, $shopId);

        if (!$hmacKey) {
            return self::VALIDATION_FAILED_HMAC_KEY_NOT_FOUND;
        }

        // Generate the expected signature.
        $expectedSignature = base64_encode(call_user_func('hash_hmac', 'sha256', $body, $hmacKey, true)); // @phpstan-ignore-line

        // Check if the expected signature is present.
        if ($expectedSignature !== $headers['SIGNATURE']) {
            return self::VALIDATION_FAILED_SIGNATURE_MISMATCH;
        }

        return self::VALIDATION_SUCCEEDED;
    }

    /**
     * @param int $orderId
     * @param string $content
     * @param string $type
     * @return void
     * @throws PrestaShopException
     */
    private function createPaygreenOrderMessage($orderId, $content, $type)
    {
        // @phpstan-ignore-next-line
        $paygreenOrderMessage = new PaygreenPaymentOrderMessageModel();
        $paygreenOrderMessage->setOrderId($orderId);
        $paygreenOrderMessage->setContent($content);
        $paygreenOrderMessage->setType($type);
        $timestamp = (new DateTime())->getTimestamp();
        $paygreenOrderMessage->setCreatedAt($timestamp);
        $paygreenOrderMessage->setUpdatedAt($timestamp);
        $paygreenOrderMessage->save();
    }

    /**
     * @param int $cartId
     * @return Cart|null
     * @throws Exception
     */
    private function getCart($cartId)
    {
        $shopId = PaygreenPayment::getShopIdContext(); /** @phpstan-ignore-line */
        $cart = new Cart($cartId);

        if (!Validate::isLoadedObject($cart)) {
            $cart = null;
        }

        if ($cart !== null && $cart->id_shop != $shopId) {
            $cart = null;
        }

        return $cart;
    }

    /**
     * @param int $amount
     *
     * @return float
     */
    private function formatAmount($amount)
    {
        return round($amount / 100, 2);
    }
}
