<?php
/**
 * Copyright since 2022 PayGreen
 * NOTICE OF LICENSE
 * This source file is subject to the MIT License X11
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/mit-license.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@paygreen.fr so we can send you a copy immediately.
 * @author    PayGreen <contact@paygreen.fr>
 * @copyright 2014 - 2022 Watt Is It
 * @license   https://opensource.org/licenses/mit-license.php MIT License X11
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once __DIR__ . '/vendor/autoload.php';
require_once __DIR__ . '/classes/PaygreenPaymentOrderModel.php';
require_once __DIR__ . '/classes/PaygreenPaymentOrderMessageModel.php';
require_once __DIR__ . '/src/PaygreenPaymentUtils.php';
require_once __DIR__ . '/src/PaygreenPaymentForbiddenAccessException.php';
require_once __DIR__ . '/src/PaygreenPaymentListenerCreationException.php';

use Paygreen\Sdk\Core\Environment as SdkEnvironment;
use Paygreen\Sdk\Payment\V3\Model\Listener as SdkListener;
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

class PaygreenPayment extends PaymentModule
{
    public static $moduleVersion = '1.2.9';

    public static $webhook_events = [
        'payment_order.authorized',
        'payment_order.expired',
        'payment_order.canceled',
        'payment_order.refused',
        'payment_order.error',
        'payment_order.refunded',
        'operation.refund',
    ];

    public $errors = [];

    public $warning = [];

    public $success = [];

    /**
     * List of ModuleFrontController used in this Module
     * Module::install() register it, after that you can edit it in BO (for rewrite if needed).
     * @var array
     */
    public $controllers = [
        'createPaymentOrder',
        'failure',
        'success',
        'webhook',
    ];

    public function __construct()
    {
        $this->name = 'paygreenpayment';
        $this->tab = 'payments_gateways';
        $this->version = '1.2.9';
        $this->author = 'PayGreen';
        $this->need_instance = 0;
        $this->module_key = '1c40f1548475021d61ae67e2eb26fd6a';
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('PayGreen Payment');
        $this->description = $this->l('PayGreen is a 100% French online payment solution that allows you to accept and manage payments on your e-commerce in a simple and efficient way. We are the first payment platform focused on sustainable development.');

        $this->confirmUninstall = $this->l('Do you really want to remove the PayGreen payment method?');

        $this->limited_currencies = ['EUR'];

        $this->ps_versions_compliancy = ['min' => '1.7', 'max' => _PS_VERSION_];
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update.
     */
    public function install()
    {
        if (!extension_loaded('curl')) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module.');

            return false;
        }

        include __DIR__ . '/sql/install.php';

        $result = parent::install();

        if (!PaygreenPaymentUtils::isVersion8()) {
            $result &= $this->registerHook('backOfficeHeader');

            if (version_compare(PaygreenPaymentUtils::getPrestashopVersion(), '1.7.7', '>=')) {
                $result &= $this->registerHook('displayAdminOrderMain');
            } else {
                $result &= $this->registerHook('displayAdminOrderLeft');
            }
        } else {
            $result &= $this->registerHook('displayBackOfficeHeader');
            $result &= $this->registerHook('displayAdminOrderMain');
        }

        $result &= $this->registerHook('payment');
        $result &= $this->registerHook('paymentOptions');
        $result &= $this->registerHook('displayOrderConfirmation');

        if (PaygreenPaymentUtils::isDevMode()) {
            Configuration::updateGlobalValue('PAYGREEN_PAYMENT_DETAILED_LOGS', 1);
        }

        Configuration::updateGlobalValue('PAYGREEN_PAYMENT_SHIPPING_COST_EXCLUSION', 1);

        return $result;
    }

    public function uninstall()
    {
        Configuration::deleteByName('PAYGREEN_PAYMENT_SHOP_ID');
        Configuration::deleteByName('PAYGREEN_PAYMENT_ENVIRONMENT');
        Configuration::deleteByName('PAYGREEN_PAYMENT_PUBLIC_KEY');
        Configuration::deleteByName('PAYGREEN_PAYMENT_SECRET_KEY');
        Configuration::deleteByName('PAYGREEN_PAYMENT_TOKEN_EXPIRE_AT');
        Configuration::deleteByName('PAYGREEN_PAYMENT_TOKEN');
        Configuration::deleteByName('PAYGREEN_PAYMENT_ENABLED_PAYMENT_METHODS');
        Configuration::deleteByName('PAYGREEN_PAYMENT_SUB_SHOP_ID');
        Configuration::deleteByName('PAYGREEN_PAYMENT_DETAILED_LOGS');
        Configuration::deleteByName('PAYGREEN_PAYMENT_BUTTON_LABEL');
        Configuration::deleteByName('PAYGREEN_PAYMENT_SHIPPING_COST_EXCLUSION');

        include __DIR__ . '/sql/uninstall.php';

        return parent::uninstall();
    }

    public function getContent()
    {
        $isShopContext = $this->isShopContext();
        $this->context->smarty->assign('is_shop_context', $isShopContext);

        if ($isShopContext) {
            $shopGroupId = self::getShopGroupIdContext();
            $shopId = self::getShopIdContext();

            $this->context->smarty->assign('display_eligible_categories_tab', false);

            if (((bool) Tools::isSubmit('submitPaygreenPaymentConfigModule')) && $this->validationPaygreenPaymentConfigForm()) {
                $this->postConfigProcess();
            }

            if ((bool) Tools::isSubmit('submitPaygreenPaymentEligibleCategoriesFOODModule')) {
                $this->postEligibleCategoriesProcess('FOOD');
                $this->context->smarty->assign('display_eligible_categories_tab', true);
            }

            if ((bool) Tools::isSubmit('submitPaygreenPaymentEligibleCategoriesTRAVELModule')) {
                $this->postEligibleCategoriesProcess('TRAVEL');
                $this->context->smarty->assign('display_eligible_categories_tab', true);
            }

            if ((bool) Tools::isSubmit('submitPaygreenPaymentShippingCostExclusionModule')) {
                $this->postShippingCostExclusionProcess();
                $this->context->smarty->assign('display_eligible_categories_tab', true);
            }

            if ((bool) Tools::isSubmit('pgaction') && Tools::getValue('pgaction') === 'repair_webhook') {
                try {
                    $this->registerPaymentValidationWebhook();
                } catch (Exception $exception) {
                    $this->errors[] = $exception->getMessage();
                }
            }

            $this->context->smarty->assign('module_dir', $this->_path);
            $this->context->smarty->assign('config_form', $this->renderConfigForm());
            $this->context->smarty->assign('is_connected', $this->isConnected());

            if (Configuration::get('PAYGREEN_PAYMENT_TOKEN', null, $shopGroupId, $shopId)) {
                $this->context->smarty->assign('is_payment_webhook_well_configured', $this->isPaymentWebhookWellConfigured());
                $this->context->smarty->assign('payment_webhook_repair_link', $this->buildWebhookRepairLink());
                $this->context->smarty->assign('eligible_categories_food_form', $this->renderEligibleCategoriesForm('FOOD'));
                $this->context->smarty->assign('eligible_categories_travel_form', $this->renderEligibleCategoriesForm('TRAVEL'));
                $this->context->smarty->assign('exclude_shipping_cost_form', $this->renderShippingCostExclusionForm());
            }
        } else {
            $this->warning[] = $this->l('The PayGreen module is only fully configurable at the store level.');
        }

        if (count($this->warning)) {
            $this->context->smarty->assign('warnings', $this->warning);
        }

        if (count($this->success) && !count($this->errors)) {
            $this->context->smarty->assign('success', $this->success);
        }

        if (count($this->errors)) {
            $this->context->smarty->assign('errors', $this->errors);
        }

        return $this->context->smarty->fetch($this->local_path . 'views/templates/admin/main.tpl');
    }

    public function renderConfigForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitPaygreenPaymentConfigModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFormValues(), // Add values for your inputs
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];

        return $helper->generateForm([$this->getConfigForm()]);
    }

    /**
     * Save form data.
     */
    public function postConfigProcess()
    {
        $shopGroupId = self::getShopGroupIdContext();
        $shopId = self::getShopIdContext();
        $formValues = $this->getConfigFormValues();

        foreach (array_keys($formValues) as $key) {
            if ($key === 'PAYGREEN_PAYMENT_BUTTON_LABEL') {
                $this->saveMultilanguageFieldValues($key);
            } else {
                Configuration::updateValue($key, Tools::getValue($key), false, $shopGroupId, $shopId);
            }
        }

        Configuration::deleteByName('PAYGREEN_PAYMENT_TOKEN_EXPIRE_AT');
        Configuration::deleteByName('PAYGREEN_PAYMENT_TOKEN');

        try {
            $client = PaygreenPaymentUtils::getPaygreenClient();

            $response = $client->getPublicKey(Configuration::get('PAYGREEN_PAYMENT_PUBLIC_KEY', false, $shopGroupId, $shopId));

            $data = json_decode($response->getBody()->getContents());

            if (!isset($data->data) || null === $data->data || null !== $data->data->revoked_at) {
                $this->errors[] = $this->l(
                    'Authentication failed. Please check your public key.',
                    $this->name
                );
            }

            $this->registerPaymentValidationWebhook();
        } catch (PaygreenPaymentForbiddenAccessException $exception) {
            $this->errors[] = $this->l($exception->getMessage(), $this->name);
        } catch (PaygreenPaymentListenerCreationException $exception) {
            $this->errors[] = $exception->getMessage();
        } catch (Exception $exception) {
            $this->errors[] = $this->l(
                'Authentication failed. Please check your credentials or the selected environment.',
                $this->name
            );
        }
    }

    /**
     * @param $type
     *
     * @return string
     *
     * @throws PrestaShopException
     */
    public function renderEligibleCategoriesForm($type)
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitPaygreenPaymentEligibleCategories' . $type . 'Module';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = [
            'fields_value' => $this->getEligibleCategoriesFormValues($type),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];

        return $helper->generateForm([$this->getEligibleCategoriesForm($type)]);
    }

    /**
     * @param string $type
     */
    public function postEligibleCategoriesProcess($type)
    {
        $shopGroupId = self::getShopGroupIdContext();
        $shopId = self::getShopIdContext();

        $values = [
            'PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type => Tools::getValue('PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type),
        ];

        Configuration::updateValue(
            'PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type,
            json_encode($values),
            false,
            $shopGroupId,
            $shopId
        );

        Configuration::updateValue(
            'PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type . '_ALLOW_ALL',
            Tools::getValue('PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type . '_ALLOW_ALL'),
            false,
            $shopGroupId,
            $shopId
        );
    }

    public function renderShippingCostExclusionForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitPaygreenPaymentShippingCostExclusionModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = [
            'fields_value' => $this->getShippingCostExclusionFormValues(), // Add values for your inputs
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];

        return $helper->generateForm([$this->getShippingCostExclusionForm()]);
    }

    public function postShippingCostExclusionProcess()
    {
        $shopGroupId = self::getShopGroupIdContext();
        $shopId = self::getShopIdContext();
        $formValues = $this->getShippingCostExclusionFormValues();

        foreach (array_keys($formValues) as $key) {
            Configuration::updateValue($key, Tools::getValue($key), false, $shopGroupId, $shopId);
        }
    }

    /**
     * Get current ShopId according to activate multishop feature.
     * @return int|null
     */
    public static function getShopIdContext()
    {
        if (Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE')) {
            return Context::getContext()->shop->id;
        }

        return Configuration::get('PS_SHOP_DEFAULT');
    }

    /**
     * Get current ShopGroupId according to activate multishop feature.
     * @return int|null
     */
    public static function getShopGroupIdContext()
    {
        if (Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE')) {
            return Context::getContext()->shop->id_shop_group;
        }

        return Configuration::get('PS_SHOP_DEFAULT');
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') === $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }

    /**
     * Return payment options available for PS 1.7+.
     * @param array Hook parameters
     * @param mixed $params
     * @return array|null
     * @throws Exception
     */
    public function hookPaymentOptions($params)
    {
        $options = [];

        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        if ($this->isConnected()) {
            $option = new PaymentOption();
            $option
                ->setCallToActionText($this->getPaymentButtonLabel($params['cookie']->id_lang))
                ->setAction($this->context->link->getModuleLink($this->name, 'createPaymentOrder'))
            ;
            $options[] = $option;
        }

        return $options;
    }

    /**
     * @param $params
     * @return false|string|void
     */
    public function hookOrderConfirmation($params)
    {
        $order = $params['order'];

        if (!$this->active || $order->module !== $this->name) {
            return;
        }

        $this->context->smarty->assign(['orderReference' => pSQL($order->reference)]);

        return $this->display(__FILE__, 'views/templates/front/order-confirmation.tpl');
    }

    /**
     * @param $params
     * @return string|void
     */
    public function hookDisplayAdminOrderMain($params)
    {
        try {
            $order = new Order($params['id_order']);

            if (Validate::isLoadedObject($order)) {
                $paygreenOrder = new PaygreenPaymentOrderModel();
                $paygreenOrder->getByOrderId($order->id);

                if (!empty($paygreenOrder->getPaymentOrderId())) {
                    $this->context->smarty->assign([
                        'payment_order_id' => $paygreenOrder->getPaymentOrderId(),
                    ]);
                }

                $paygreenOrderMessages = PaygreenPaymentOrderMessageModel::getAllByOrderId($order->id);
                $messages = [];

                /** @var PaygreenPaymentOrderMessageModel $paygreenOrderMessage */
                foreach ($paygreenOrderMessages as $paygreenOrderMessage) {
                    if ($paygreenOrderMessage->getType() === 'operation.refund') {
                        $messages[] = $this->l('Refund successfully executed. Amount refunded : ') . $paygreenOrderMessage->getContent();
                    } else {
                        $messages[] = $paygreenOrderMessage->getContent();
                    }
                }

                $this->context->smarty->assign([
                    'payment_order_messages' => $messages,
                ]);

                return $this->display(__FILE__, 'views/templates/hook/displayAdminOrderMain.tpl');
            }
        } catch (Exception $exception) {
            PaygreenPaymentUtils::addLog('PaygreenPayment::hookDisplayAdminOrderMain - An error occurred in displayAdminOrderMain hook', 3);
        }
    }

    /**
     * @param $params
     * @return string|void
     */
    public function hookDisplayAdminOrderLeft($params)
    {
        try {
            $order = new Order($params['id_order']);

            if (Validate::isLoadedObject($order)) {
                $paygreenOrder = new PaygreenPaymentOrderModel();
                $paygreenOrder->getByOrderId($order->id);

                if (!empty($paygreenOrder->getPaymentOrderId())) {
                    $this->context->smarty->assign([
                        'payment_order_id' => $paygreenOrder->getPaymentOrderId(),
                    ]);
                }

                $paygreenOrderMessages = PaygreenPaymentOrderMessageModel::getAllByOrderId($order->id);
                $messages = [];

                /** @var PaygreenPaymentOrderMessageModel $paygreenOrderMessage */
                foreach ($paygreenOrderMessages as $paygreenOrderMessage) {
                    if ($paygreenOrderMessage->getType() === 'operation.refund') {
                        $messages[] = $this->l('Refund successfully executed. Amount refunded : ') . $paygreenOrderMessage->getContent();
                    } else {
                        $messages[] = $paygreenOrderMessage->getContent();
                    }
                }

                $this->context->smarty->assign([
                    'payment_order_messages' => $messages,
                ]);

                return $this->display(__FILE__, 'views/templates/hook/displayAdminOrderLeft.tpl');
            }
        } catch (Exception $exception) {
            PaygreenPaymentUtils::addLog('PaygreenPayment::hookDisplayAdminOrderLeft - An error occurred in displayAdminOrderLeft hook', 3);
        }
    }

    public function checkCurrency($cart)
    {
        $cartCurrency = new Currency($cart->id_currency);

        return 'EUR' === strtoupper($cartCurrency->iso_code);
    }

    protected function getConfigForm()
    {
        $options = [
            [
                'id_option' => SdkEnvironment::ENVIRONMENT_SANDBOX,
                'name' => $this->l('Sandbox'),
            ],
            [
                'id_option' => SdkEnvironment::ENVIRONMENT_PRODUCTION,
                'name' => $this->l('Production'),
            ],
        ];

        if (PaygreenPaymentUtils::isDevMode()) {
            $options[] = [
                'id_option' => SdkEnvironment::ENVIRONMENT_RECETTE,
                'name' => $this->l('Recette'),
            ];
        }

        return [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'select',
                        'label' => $this->l('Environment'),
                        'name' => 'PAYGREEN_PAYMENT_ENVIRONMENT',
                        'desc' => $this->l('Use this module in production.'),
                        'required' => true,
                        'options' => [
                            'query' => $options,
                            'id' => 'id_option',
                            'name' => 'name',
                        ],
                    ],
                    [
                        'col' => 4,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-user"></i>',
                        'name' => 'PAYGREEN_PAYMENT_SHOP_ID',
                        'label' => $this->l('Shop identifier'),
                        'required' => true,
                    ],
                    [
                        'col' => 4,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-key"></i>',
                        'name' => 'PAYGREEN_PAYMENT_PUBLIC_KEY',
                        'label' => $this->l('Public key'),
                        'required' => true,
                    ],
                    [
                        'col' => 4,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-key"></i>',
                        'name' => 'PAYGREEN_PAYMENT_SECRET_KEY',
                        'label' => $this->l('Secret key'),
                        'required' => true,
                    ],
                    [
                        'col' => 4,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-user"></i>',
                        'name' => 'PAYGREEN_PAYMENT_SUB_SHOP_ID',
                        'label' => $this->l('Sub-Shop identifier'),
                        'desc' => $this->l('If this store is a sub-shop and you want to create payments in its name, add its id.'),
                        'required' => false,
                    ],
                    [
                        'col' => 4,
                        'type' => 'switch',
                        'prefix' => '<i class="icon icon-user"></i>',
                        'name' => 'PAYGREEN_PAYMENT_DETAILED_LOGS',
                        'is_bool' => true,
                        'label' => $this->l('Detailed logs'),
                        'desc' => $this->l('Please note that activating detailed logs will significantly increase the weight of logs saved in the database. This option should only be activated for bug resolution.'),
                        'required' => false,
                        'values' => [
                            [
                                'id' => 'PAYGREEN_PAYMENT_DETAILED_LOGS_TRUE',
                                'value' => 1,
                            ],
                            [
                                'id' => 'PAYGREEN_PAYMENT_DETAILED_LOGS_FALSE',
                                'value' => 0,
                            ],
                        ],
                    ],
                    [
                        'col' => 4,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-globe"></i>',
                        'name' => 'PAYGREEN_PAYMENT_BUTTON_LABEL',
                        'label' => $this->l('Payment button title'),
                        'desc' => $this->l('Allows you to change the title of the payment button that will be visible to your customers.'),
                        'required' => true,
                        'lang' => true,
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        $shopGroupId = self::getShopGroupIdContext();
        $shopId = self::getShopIdContext();

        return [
            'PAYGREEN_PAYMENT_SHOP_ID' => Configuration::get('PAYGREEN_PAYMENT_SHOP_ID', null, $shopGroupId, $shopId),
            'PAYGREEN_PAYMENT_ENVIRONMENT' => Configuration::get('PAYGREEN_PAYMENT_ENVIRONMENT', null, $shopGroupId, $shopId),
            'PAYGREEN_PAYMENT_PUBLIC_KEY' => Configuration::get('PAYGREEN_PAYMENT_PUBLIC_KEY', null, $shopGroupId, $shopId),
            'PAYGREEN_PAYMENT_SECRET_KEY' => Configuration::get('PAYGREEN_PAYMENT_SECRET_KEY', null, $shopGroupId, $shopId),
            'PAYGREEN_PAYMENT_SUB_SHOP_ID' => Configuration::get('PAYGREEN_PAYMENT_SUB_SHOP_ID', null, $shopGroupId, $shopId),
            'PAYGREEN_PAYMENT_DETAILED_LOGS' => Configuration::get('PAYGREEN_PAYMENT_DETAILED_LOGS', null, $shopGroupId, $shopId),
            'PAYGREEN_PAYMENT_BUTTON_LABEL' => $this->getMultilanguageFieldValues('PAYGREEN_PAYMENT_BUTTON_LABEL', 'Pay with PayGreen'),
        ];
    }

    protected function validationPaygreenPaymentConfigForm()
    {
        $formConfigValues = [
            'PAYGREEN_PAYMENT_SHOP_ID' => $this->l('Store identifier'),
            'PAYGREEN_PAYMENT_PUBLIC_KEY' => $this->l('Public key'),
            'PAYGREEN_PAYMENT_SECRET_KEY' => $this->l('Secret key'),
        ];

        $result = true;

        foreach ($formConfigValues as $key => $value) {
            $configFormValue = Tools::getValue($key);

            if (empty($configFormValue) || !Validate::isGenericName($configFormValue)) {
                $this->errors[] = $this->l(
                    'Authentication failed. This field is required: ',
                    $this->name
                ) . $value;
                $result = false;
            }
        }

        return $result;
    }

    protected function getShippingCostExclusionForm()
    {
        return [
            'form' => [
                'legend' => [
                    'title' => $this->l('Exclusion of shipping costs'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'switch',
                        'label' => $this->l('Exclusion of shipping costs'),
                        'name' => 'PAYGREEN_PAYMENT_SHIPPING_COST_EXCLUSION',
                        'desc' => $this->l('By checking this box, shipping costs will be excluded from the calculation of eligible amounts.'),
                        'required' => false,
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'shipping_cost_exclusion_yes',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'shipping_cost_exclusion_no',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
    }

    protected function getShippingCostExclusionFormValues()
    {
        $shopGroupId = self::getShopGroupIdContext();
        $shopId = self::getShopIdContext();

        return [
            'PAYGREEN_PAYMENT_SHIPPING_COST_EXCLUSION' => Configuration::get('PAYGREEN_PAYMENT_SHIPPING_COST_EXCLUSION', null, $shopGroupId, $shopId),
        ];
    }

    /**
     * @return bool
     */
    protected function isConnected()
    {
        $shopGroupId = self::getShopGroupIdContext();
        $shopId = self::getShopIdContext();

        $pgShopId = Configuration::get('PAYGREEN_PAYMENT_SHOP_ID', null, $shopGroupId, $shopId);
        $pgSecretKey = Configuration::get('PAYGREEN_PAYMENT_SECRET_KEY', null, $shopGroupId, $shopId);
        $pgEnvironment = Configuration::get('PAYGREEN_PAYMENT_ENVIRONMENT', null, $shopGroupId, $shopId);

        if (empty($pgShopId) || empty($pgSecretKey) || empty($pgEnvironment)) {
            return false;
        }

        try {
            PaygreenPaymentUtils::getPaygreenClient();
        } catch (Exception $exception) {
            return false;
        }

        return true;
    }

    /**
     * @param $type
     *
     * @return array[]
     *
     * @throws Exception
     */
    protected function getEligibleCategoriesForm($type)
    {
        $shopGroupId = self::getShopGroupIdContext();
        $shopId = self::getShopIdContext();

        $eligibleCategories = json_decode(Configuration::get('PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type, null, $shopGroupId, $shopId), true);

        if (empty($eligibleCategories) || !$eligibleCategories['PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type]) {
            $options = [];
        } else {
            $options = [$eligibleCategories['PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type]];
        }

        switch ($type) {
            case 'FOOD':
                $title = $this->l('Meal voucher eligible categories');
                $description = $this->l('Selected categories will be eligible for meal voucher payment methods.');

                break;

            case 'TRAVEL':
                $title = $this->l('Vacation voucher eligible categories');
                $description = $this->l('Selected categories will be eligible for vacation voucher payment methods.');

                break;

            default:
                throw new Exception("Unknown type '$type'.");
        }

        return [
            'form' => [
                'legend' => [
                    'title' => $title,
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'type' => 'switch',
                        'label' => $this->l('Make all categories eligible'),
                        'name' => 'PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type . '_ALLOW_ALL',
                        'desc' => $this->l('By checking this box, all product categories will be eligible. Regardless of the categories selected below.'),
                        'required' => false,
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'paygreen_payment_eligible_categories_' . $type . '_allow_all_yes',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'paygreen_payment_eligible_categories_' . $type . '_allow_all_no',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'categories',
                        'label' => $this->l('Choose categories'),
                        'name' => 'PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type,
                        'desc' => $description,
                        'required' => false,
                        'tree' => [
                            'id' => 'paygreen_payment_eligible_categories_' . $type,
                            'use_checkbox' => true,
                            'use_search' => true,
                            'selected_categories' => $options ? $options[0] : null,
                        ],
                    ],
                ],
                'reset' => [
                    'title' => $this->l('Reset'),
                    'icon' => 'icon-reset',
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
    }

    /**
     * @param string $type
     * @return array
     */
    protected function getEligibleCategoriesFormValues($type)
    {
        $shopGroupId = self::getShopGroupIdContext();
        $shopId = self::getShopIdContext();

        return [
            'PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type => Configuration::get('PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type, null, $shopGroupId, $shopId),
            'PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type . '_ALLOW_ALL' => Configuration::get('PAYGREEN_PAYMENT_ELIGIBLE_CATEGORIES_' . $type . '_ALLOW_ALL', null, $shopGroupId, $shopId),
        ];
    }

    /**
     * @return bool
     */
    private function isShopContext()
    {
        $shop = $this->context->shop;

        return Shop::CONTEXT_SHOP === $shop::getContext();
    }

    /**
     * @param string $urlA
     * @param string $urlB
     * @return bool
     */
    private function isForSameShop($urlA, $urlB)
    {
        $paramsA = explode('&', parse_url($urlA, PHP_URL_QUERY));
        $paramsB = explode('&', parse_url($urlB, PHP_URL_QUERY));

        return in_array('id_shop=' . self::getShopIdContext(), array_intersect($paramsA, $paramsB));
    }

    /**
     * @return bool
     *
     * @throws PaygreenPaymentListenerCreationException
     */
    private function registerPaymentValidationWebhook()
    {
        $result = false;

        $shopGroupId = self::getShopGroupIdContext();
        $shopId = self::getShopIdContext();
        $client = PaygreenPaymentUtils::getPaygreenClient();
        $paygreenShopId = Configuration::get('PAYGREEN_PAYMENT_SHOP_ID', null, $shopGroupId, $shopId);
        $isUrlRewriting = (int) Configuration::get('PS_REWRITING_SETTINGS', null, $shopGroupId, $shopId) === 1;
        $validationUrl = $this->buildValidationWebhookUrl($isUrlRewriting);
        $baseUrl = $this->context->link->getBaseLink($shopId, true);

        $response = $client->listListener($paygreenShopId);

        if (PaygreenPaymentUtils::isValidResponse($response->getStatusCode())) {
            $responseData = json_decode($response->getBody()->getContents(), true);

            if ($responseData['pagination']['totalResults'] >= 10) {
                throw new PaygreenPaymentListenerCreationException($this->l('The limit of 10 listeners for a store has been reached. The listener subscription has failed. Go to the PayGreen backoffice to manage your listeners.'));
            }

            $listeners = $responseData['data'];

            if (!empty($listeners)) {
                foreach ($listeners as $listener) {
                    // Removing listeners with the old format
                    if ($this->isLegacyListener($listener['url'])) {
                        $deletionResult = $this->deleteListener($listener['id']);

                        if ($deletionResult) {
                            continue;
                        }
                    }

                    // Delete listeners without rewriting url if option enabled
                    if ('webhook' === $listener['type']
                        && $listener['shop_id'] === $paygreenShopId
                        && $this->isForSameShop($listener['url'], $validationUrl)
                        && (preg_match('/(paygreenpayment)+/', $listener['url']) && preg_match('/(webhook)+/', $listener['url']))
                        && ($isUrlRewriting && preg_match('/(index\.php)/', $listener['url']))
                        && parse_url($baseUrl, PHP_URL_HOST) === parse_url($listener['url'], PHP_URL_HOST)
                    ) {
                        $deletionResult = $this->deleteListener($listener['id']);

                        if ($deletionResult) {
                            continue;
                        }
                    }

                    // Delete listeners with url rewriting if the option has been disabled
                    if ('webhook' === $listener['type']
                        && $listener['shop_id'] === $paygreenShopId
                        && $this->isForSameShop($listener['url'], $validationUrl)
                        && (preg_match('/(paygreenpayment)+/', $listener['url']) && preg_match('/(webhook)+/', $listener['url']))
                        && (!$isUrlRewriting && !preg_match('/(index\.php)/', $listener['url']))
                        && parse_url($baseUrl, PHP_URL_HOST) === parse_url($listener['url'], PHP_URL_HOST)
                    ) {
                        $deletionResult = $this->deleteListener($listener['id']);

                        if ($deletionResult) {
                            continue;
                        }
                    }

                    // Delete listener with wrong events
                    if ('webhook' === $listener['type']
                        && $listener['shop_id'] === $paygreenShopId
                        && $this->isForSameShop($listener['url'], $validationUrl)
                        && $listener['url'] === $validationUrl
                        && $listener['events'] !== self::$webhook_events
                    ) {
                        $deletionResult = $this->deleteListener($listener['id']);

                        if ($deletionResult) {
                            continue;
                        }
                    }

                    // Delete disabled listener
                    if ('webhook' === $listener['type']
                        && $listener['shop_id'] === $paygreenShopId
                        && $this->isForSameShop($listener['url'], $validationUrl)
                        && $listener['url'] === $validationUrl
                        && $listener['events'] === self::$webhook_events
                        && $listener['status'] === 'disabled'
                    ) {
                        $deletionResult = $this->deleteListener($listener['id']);

                        if ($deletionResult) {
                            continue;
                        }
                    }

                    if ('webhook' === $listener['type']
                        && $listener['shop_id'] === $paygreenShopId
                        && $this->isForSameShop($listener['url'], $validationUrl)
                        && $listener['url'] === $validationUrl
                        && $listener['events'] === self::$webhook_events
                    ) {
                        $result = true;
                        Configuration::updateValue('PAYGREEN_LISTENER_HMAC_KEY', $listener['hmac_key'], false, $shopGroupId, $shopId);
                    }
                }
            }

            if (!$result) {
                $listener = new SdkListener();
                $listener->setUrl($validationUrl);
                $listener->setEvents(self::$webhook_events);
                $listener->setType('webhook');

                $response = PaygreenPaymentUtils::getPaygreenClient()->createListener($listener);

                if (PaygreenPaymentUtils::isValidResponse($response->getStatusCode())) {
                    $result = true;
                    $responseData = json_decode($response->getBody()->getContents(), true);
                    $listeners = $responseData['data'];
                    Configuration::updateValue('PAYGREEN_LISTENER_HMAC_KEY', $listeners['hmac_key'], false, $shopGroupId, $shopId);
                } else {
                    PaygreenPaymentUtils::addLog('PaygreenPayment::registerPaymentValidationWebhook- Listener creation has failed. Response: ' . print_r($response->getBody()->getContents(), true), 3);

                    throw new PaygreenPaymentListenerCreationException($this->l('Payment validation listener creation has failed.'));
                }
            }
        } else {
            PaygreenPaymentUtils::addLog('PaygreenPayment::registerPaymentValidationWebhook - Failed to get listeners. Response: ' . print_r($response->getBody()->getContents(), true), 3);

            throw new PaygreenPaymentListenerCreationException($this->l('Payment validation listener creation has failed.'));
        }

        return $result;
    }

    /**
     * @param string $url
     * @return bool
     * @throws Exception
     */
    private function isLegacyListener($url)
    {
        if (preg_match('/(paygreenpayment)+/', $url) && preg_match('/(webhook)+/', $url)) {
            $params = explode('&', parse_url($url, PHP_URL_QUERY));

            foreach ($params as $param) {
                if (preg_match('/(id_shop=)+/', $param)) {
                    return false;
                }
            }

            return true;
        }

        return false;
    }

    /**
     * @param $listenerId
     * @return true
     * @throws PaygreenPaymentListenerCreationException
     */
    private function deleteListener($listenerId)
    {
        $response = PaygreenPaymentUtils::getPaygreenClient()->deleteListener($listenerId);

        if (!PaygreenPaymentUtils::isValidResponse($response->getStatusCode())) {
            PaygreenPaymentUtils::addLog('PaygreenPayment::deleteListener - Listener deletion has failed : ' . $listenerId, 3);

            throw new PaygreenPaymentListenerCreationException($this->l('The legacy listener deletion has failed : ' . $listenerId));
        }

        return true;
    }

    /**
     * @param bool $isUrlRewriting
     * @return string
     * @throws PrestaShopException
     */
    private function buildValidationWebhookUrl($isUrlRewriting)
    {
        $shopId = self::getShopIdContext();
        $idLang = Context::getContext()->language->id;
        $url = $this->context->link->getBaseLink($shopId, true);
        $params['module'] = $this->name;
        $params['controller'] = 'webhook';
        $validationUrl = $url . Dispatcher::getInstance()->createUrl('module', $idLang, $params, $isUrlRewriting, '', $shopId);

        if (empty(parse_url($validationUrl, PHP_URL_QUERY))) {
            $validationUrl .= '?id_shop=' . $shopId;
        } else {
            $validationUrl .= '&id_shop=' . $shopId;
        }

        if (getenv('PAYGREEN_DEBUG')) {
            $validationUrl = str_replace('http://', 'https://', $validationUrl);
        }

        return $validationUrl;
    }

    /**
     * @return string
     * @throws PrestaShopException
     */
    private function buildWebhookRepairLink()
    {
        $link = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name . '&pgaction=repair_webhook';
        $token = Tools::getAdminTokenLite('AdminModules');

        return $link . '&token=' . $token;
    }

    /**
     * @return bool
     * @throws Exception
     */
    private function isPaymentWebhookWellConfigured()
    {
        $result = [];
        $shopGroupId = self::getShopGroupIdContext();
        $shopId = self::getShopIdContext();
        $client = PaygreenPaymentUtils::getPaygreenClient();
        $paygreenShopId = Configuration::get('PAYGREEN_PAYMENT_SHOP_ID', null, $shopGroupId, $shopId);
        $isUrlRewriting = (int) Configuration::get('PS_REWRITING_SETTINGS', null, $shopGroupId, $shopId) === 1;
        $hmacKey = Configuration::get('PAYGREEN_LISTENER_HMAC_KEY', null, $shopGroupId, $shopId);
        $validationUrl = $this->buildValidationWebhookUrl($isUrlRewriting);
        $response = $client->listListener($paygreenShopId);

        if (PaygreenPaymentUtils::isValidResponse($response->getStatusCode())) {
            $responseData = json_decode($response->getBody()->getContents(), true);

            if ($responseData['pagination']['totalResults'] >= 10) {
                return false;
            }

            $listeners = $responseData['data'];

            if (!empty($listeners)) {
                foreach ($listeners as $listener) {
                    if ($this->isLegacyListener($listener['url'])) {
                        return false;
                    }

                    if ('webhook' === $listener['type']
                        && $listener['shop_id'] === $paygreenShopId
                        && $this->isForSameShop($listener['url'], $validationUrl)
                        && $listener['url'] === $validationUrl
                        && $listener['events'] === self::$webhook_events
                        && $listener['status'] === 'active'
                    ) {
                        $result[] = $listener;
                    }
                }
            }

            if (count($result) === 1) {
                if (!$isUrlRewriting) {
                    $isUrlOk = (bool) preg_match('/(index\.php)/', $result[0]['url']);
                } else {
                    $isUrlOk = (bool) !preg_match('/(index\.php)/', $result[0]['url']);
                }

                if ($isUrlOk && $result[0]['hmac_key'] === $hmacKey) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * @param string $name
     * @param string $defaultValueString
     * @return array
     */
    private function getMultilanguageFieldValues($name, $defaultValueString)
    {
        $values = [];
        $shopGroupId = self::getShopGroupIdContext();
        $shopId = self::getShopIdContext();

        $languages = $this->context->controller->getLanguages();
        $frLangLocale = null;
        $frLangValue = null;
        $enLangLocale = null;
        $enLangValue = null;

        foreach ($languages as $language) {
            $nameWithIdLang = $name . '_' . $language['id_lang'];
            $customValue = Configuration::get($nameWithIdLang, null, $shopGroupId, $shopId);
            $value = !empty($customValue) ? $customValue : null;

            switch ($language['iso_code']) {
                case 'fr':
                    $frLangLocale = $language['locale'];
                    $frLangValue = $value;
                    break;
                case 'en':
                    $enLangLocale = $language['locale'];
                    $enLangValue = $value;
                    break;
            }
        }

        foreach ($languages as $language) {
            $languageId = (int) $language['id_lang'];
            $nameWithIdLang = $name . '_' . $language['id_lang'];
            $customValue = Configuration::get($nameWithIdLang, null, $shopGroupId, $shopId);

            if (!empty($customValue)) {
                $values[$languageId] = $customValue;
            } elseif ($language['iso_code'] === 'fr') {
                if (empty($frLangValue) && !empty($frLangLocale)) {
                    $frLangValue = $this->l($defaultValueString, false, $frLangLocale);
                } else {
                    $frLangValue = '';
                }

                $values[$languageId] = $frLangValue;
            } else {
                if (empty($enLangValue) && !empty($enLangLocale)) {
                    $enLangValue = $this->l($defaultValueString, false, $enLangLocale);
                } else {
                    $enLangValue = '';
                }

                $values[$languageId] = $enLangValue;
            }
        }

        return $values;
    }

    /**
     * @param string $name
     */
    private function saveMultilanguageFieldValues($name)
    {
        $shopGroupId = self::getShopGroupIdContext();
        $shopId = self::getShopIdContext();

        $languages = $this->context->controller->getLanguages();

        foreach ($languages as $language) {
            $nameWithIdLang = $name . '_' . $language['id_lang'];
            Configuration::updateValue($nameWithIdLang, Tools::getValue($nameWithIdLang), false, $shopGroupId, $shopId);
        }
    }

    /**
     * @param int|string $langId
     * @return string
     * @throws Exception
     */
    private function getPaymentButtonLabel($langId)
    {
        $shopGroupId = self::getShopGroupIdContext();
        $shopId = self::getShopIdContext();

        $isoCode = Language::getIsoById((int) $langId);
        $locale = Language::getLocaleByIso($isoCode);

        $key = 'PAYGREEN_PAYMENT_BUTTON_LABEL_' . (string) $langId;
        $label = Configuration::get($key, null, $shopGroupId, $shopId);

        if (!empty($label)) {
            return $label;
        } else {
            return $this->l('Pay with PayGreen', false, $locale);
        }
    }
}
