<?php
/**
 * Copyright since 2022 PayGreen
 * NOTICE OF LICENSE
 * This source file is subject to the MIT License X11
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/mit-license.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@paygreen.fr so we can send you a copy immediately.
 * @author    PayGreen <contact@paygreen.fr>
 * @copyright 2014 - 2022 Watt Is It
 * @license   https://opensource.org/licenses/mit-license.php MIT License X11
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once __DIR__ . '/../classes/PaygreenPaymentOrderModel.php';
require_once __DIR__ . '/PaygreenPaymentUtils.php';

class PaygreenPaymentOrderHelper
{
    /**
     * @param string|int $cartId
     * @param string $poId
     * @param string $context
     * @return Order|OrderCore|null
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function getOrder($cartId, $poId, $context)
    {
        $order = Order::getByCartId($cartId);

        // @phpstan-ignore-next-line
        if (!Validate::isLoadedObject($order)) {
            PaygreenPaymentUtils::addLog(strtoupper($context) . ' - PaygreenPaymentOrderHelper::getOrder - ' . $poId . ' - Order not found for cart id : ' . $cartId, 3);

            return null;
        }

        return $order;
    }

    /**
     * @param string|int $cartId
     *
     * @return bool
     */
    public static function isOrderExist($cartId)
    {
        $cacheId = 'Cart::orderExists_' . (int) $cartId;
        $result = (bool) Db::getInstance()->getValue('SELECT count(*) FROM `' . _DB_PREFIX_ . 'orders` WHERE `id_cart` = ' . (int) $cartId);
        Cache::store($cacheId, $result);

        return $result;
    }

    /**
     * @param Order $order
     *
     * @return bool
     */
    public static function isOrderPaid($order)
    {
        $orderState = $order->getCurrentOrderState();

        return $orderState && $orderState->paid;
    }

    /**
     * @param Order $order
     *
     * @return bool
     */
    public static function isOrderError($order)
    {
        return (int) $order->getCurrentState() === (int) Configuration::get('PS_OS_ERROR');
    }

    /**
     * @param Order $order
     *
     * @return bool
     */
    public static function isOrderCancel($order)
    {
        return (int) $order->getCurrentState() === (int) Configuration::get('PS_OS_CANCELED');
    }

    /**
     * NOTE: The paygreen order will not be created if it already exists.
     *
     * @param int $orderId
     * @param string $paymentOrderId
     * @return void
     * @throws PrestaShopException
     */
    public static function createPaygreenOrder($orderId, $paymentOrderId)
    {
        $paygreenOrder = new PaygreenPaymentOrderModel();
        $paygreenOrder->getByOrderId($orderId);

        if (empty($paygreenOrder->getPaymentOrderId())) {
            $paygreenOrder->setOrderId($orderId);
            $paygreenOrder->setPaymentOrderId($paymentOrderId);
            $timestamp = (new DateTime())->getTimestamp();
            $paygreenOrder->setCreatedAt($timestamp);
            $paygreenOrder->setUpdatedAt($timestamp);
            $paygreenOrder->save();
        }
    }

    /**
     * @param Module $module
     * @param Cart $cart
     * @param string $secureKey
     * @param string $status
     * @param string $poId
     * @param string $context
     * @return bool
     * @throws Exception
     */
    public static function createOrder(
        $module,
        $cart,
        $secureKey,
        $status,
        $poId,
        $context
    ) {
        if (!in_array($status, ['authorized', 'successed'])) {
            PaygreenPaymentUtils::addLog(strtoupper($context) . ' - PaygreenPaymentOrderHelper::createOrder - ' . $poId . ' - Wrong status provided : ' . $status, 3);

            return false;
        }

        try {
            PaygreenPaymentUtils::addLog(strtoupper($context) . ' - PaygreenPaymentOrderHelper::createOrder - ' . $poId . ' - validateOrder - start');

            // @phpstan-ignore-next-line
            $module->validateOrder(
                $cart->id,
                Configuration::get('PS_OS_PAYMENT'),
                self::getOrderAmount($cart),
                $module->displayName,
                null,
                [],
                $cart->id_currency,
                false,
                $secureKey
            );

            PaygreenPaymentUtils::addLog(strtoupper($context) . ' - PaygreenPaymentOrderHelper::createOrder - ' . $poId . ' - validateOrder - end');

            // @phpstan-ignore-next-line
            if (!$order = PaygreenPaymentOrderHelper::getOrder($cart->id, $poId, $context)) {
                return false;
            }

            // @phpstan-ignore-next-line
            if (!PaygreenPaymentOrderHelper::isOrderPaid($order)) {
                PaygreenPaymentUtils::addLog(strtoupper($context) . ' - PaygreenPaymentOrderHelper::createOrder - ' . $poId . ' - Incorrect order status after prestashop validation process : ' . self::getOrderStateName($order->getCurrentOrderState()), 2);

                return false;
            }
        } catch (Exception $exception) {
            PaygreenPaymentUtils::addLog(strtoupper($context) . ' - PaygreenPaymentOrderHelper::createOrder - ' . $poId . ' - Exception - ' . preg_replace("/\n/", '<br>', (string) $exception->getMessage() . '<br>' . $exception->getTraceAsString()), 3);

            return false;
        }

        return true;
    }

    /**
     * @param Cart $cart
     * @return float
     */
    public static function getOrderAmount($cart)
    {
        return (float) Tools::ps_round((float) $cart->getOrderTotal(true, Cart::BOTH), 2);
    }

    /**
     * @param OrderState $orderState
     * @return string
     */
    private static function getOrderStateName($orderState)
    {
        if (is_array($orderState->name)) {
            $name = implode('/', $orderState->name);
        } elseif (is_string($orderState->name) && !empty($orderState->name)) {
            $name = $orderState->name;
        } else {
            $name = 'Order status name not found';
        }

        return $name;
    }
}
