<?php
/**
 * Copyright since 2022 PayGreen
 * NOTICE OF LICENSE
 * This source file is subject to the MIT License X11
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/mit-license.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to contact@paygreen.fr so we can send you a copy immediately.
 * @author    PayGreen <contact@paygreen.fr>
 * @copyright 2014 - 2022 Watt Is It
 * @license   https://opensource.org/licenses/mit-license.php MIT License X11
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once __DIR__ . '/PaygreenPaymentCurlClient.php';
require_once __DIR__ . '/PaygreenPaymentForbiddenAccessException.php';

use Paygreen\Sdk\Payment\V3\Client as SdkClient;
use Paygreen\Sdk\Payment\V3\Environment as SdkEnvironment;
use Paygreen\Sdk\Payment\V3\Utils as SdkUtils;

class PaygreenPaymentUtils
{
    /**
     * @param mixed $curl_options
     *
     * @return SdkClient
     *
     * @throws Exception
     */
    public static function getPaygreenClient($curl_options = [])
    {
        $shopGroupIdContext = PaygreenPayment::getShopGroupIdContext();
        $shopIdContext = PaygreenPayment::getShopIdContext();

        $curlClient = new PaygreenPaymentCurlClient(null, null, $curl_options);
        $environment = new SdkEnvironment(
            Configuration::get('PAYGREEN_PAYMENT_SHOP_ID', null, $shopGroupIdContext, $shopIdContext),
            Configuration::get('PAYGREEN_PAYMENT_SECRET_KEY', null, $shopGroupIdContext, $shopIdContext),
            Configuration::get('PAYGREEN_PAYMENT_ENVIRONMENT', null, $shopGroupIdContext, $shopIdContext)
        );

        $environment->setApplicationName('prestashop-payment');
        $environment->setApplicationVersion(PaygreenPayment::$moduleVersion);
        $environment->setCmsName('prestashop');
        $environment->setCmsVersion(_PS_VERSION_);

        $client = new SdkClient($curlClient, $environment);

        $bearerToken = self::getBearerToken($client);

        if (null === $bearerToken) {
            throw new Exception('Client authentication failed.');
        }

        $client->setBearer($bearerToken);

        return $client;
    }

    /**
     * @param SdkClient $client
     *
     * @return string
     *
     * @throws PaygreenPaymentForbiddenAccessException
     */
    public static function authenticate($client)
    {
        $bearerToken = null;
        $response = $client->authenticate();

        $shopGroupIdContext = PaygreenPayment::getShopGroupIdContext();
        $shopIdContext = PaygreenPayment::getShopIdContext();

        if (200 === $response->getStatusCode()) {
            $data = json_decode($response->getBody()->getContents())->data;
            $decoded = SdkUtils::decodeJWT($data->token);

            Configuration::updateValue('PAYGREEN_PAYMENT_TOKEN', $data->token, false, $shopGroupIdContext, $shopIdContext);
            Configuration::updateValue('PAYGREEN_PAYMENT_TOKEN_EXPIRE_AT', $decoded->exp, false, $shopGroupIdContext, $shopIdContext);
            $bearerToken = $data->token;
        } elseif (403 === $response->getStatusCode() && SdkEnvironment::ENVIRONMENT_RECETTE === Configuration::get('PAYGREEN_PAYMENT_ENVIRONMENT', null, $shopGroupIdContext, $shopIdContext)) {
            throw new PaygreenPaymentForbiddenAccessException('Authentication failed. Please check your vpn connection.');
        } else {
            Configuration::deleteByName('PAYGREEN_PAYMENT_TOKEN_EXPIRE_AT');
            Configuration::deleteByName('PAYGREEN_PAYMENT_TOKEN');
        }

        return $bearerToken;
    }

    /**
     * @return bool
     */
    public static function isVersion8()
    {
        return (bool) version_compare(_PS_VERSION_, '8.0.0', '>=');
    }

    /**
     * @return string
     */
    public static function getPrestashopVersion()
    {
        return _PS_VERSION_;
    }

    /**
     * @param string $message
     * @param int $level (1: info, 2: warning, 3: error, 4: critical)
     * @return void
     */
    public static function addLog($message, $level = 1)
    {
        $shopGroupIdContext = PaygreenPayment::getShopGroupIdContext();
        $shopIdContext = PaygreenPayment::getShopIdContext();

        $detailedLogs = Configuration::get('PAYGREEN_PAYMENT_DETAILED_LOGS', null, $shopGroupIdContext, $shopIdContext);
        $message = (new DateTimeImmutable())->format('Y-m-d H:i:s.u') . ' - ID_SHOP : ' . $shopIdContext . ' - ' . $message;

        if ($detailedLogs || $level >= 3) {
            PrestaShopLogger::addLog($message, $level, null, null, null, true);
        }
    }

    /**
     * @param int $statusCode
     * @return bool
     */
    public static function isValidResponse($statusCode)
    {
        return $statusCode >= 200 && $statusCode <= 299;
    }

    /**
     * @param string $paymentOrderId
     * @return ?string
     * @throws Exception
     */
    public static function getPOStatus($paymentOrderId)
    {
        $client = self::getPaygreenClient();
        $response = $client->getPaymentOrder($paymentOrderId);

        if ($response->getStatusCode() === 200) {
            $responseData = json_decode($response->getBody()->getContents(), true)['data'];

            return explode('.', $responseData['status'])[1];
        } else {
            self::addLog('PaygreenPaymentUtils::getPOStatus - ' . $paymentOrderId . ' - Payment order not found', 3);

            return null;
        }
    }

    /**
     * @return bool
     */
    public static function isDevMode()
    {
        return getenv('PAYGREEN_DEBUG') == 1;
    }

    /**
     * @param SdkClient $client
     *
     * @return string
     *
     * @throws PaygreenPaymentForbiddenAccessException
     */
    private static function getBearerToken($client)
    {
        $shopGroupIdContext = PaygreenPayment::getShopGroupIdContext();
        $shopIdContext = PaygreenPayment::getShopIdContext();

        $timestamp = (new DateTime())->getTimestamp() + 10;

        if ($timestamp < Configuration::get('PAYGREEN_PAYMENT_TOKEN_EXPIRE_AT', null, $shopGroupIdContext, $shopIdContext)) {
            $bearerToken = Configuration::get('PAYGREEN_PAYMENT_TOKEN', null, $shopGroupIdContext, $shopIdContext);
        } else {
            $bearerToken = self::authenticate($client);
        }

        return $bearerToken;
    }
}
